/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.console.navigation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;


import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.portal.config.DataStorage;
import org.exoplatform.portal.config.Query;
import org.exoplatform.portal.config.UserACL;
import org.exoplatform.portal.config.UserPortalConfigService;
import org.exoplatform.portal.config.model.Application;
import org.exoplatform.portal.config.model.Page;
import org.exoplatform.portal.config.model.PageNavigation;
import org.exoplatform.portal.config.model.PageNode;
import org.exoplatform.portal.config.model.PortalConfig;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.picocontainer.Startable;

/**
 * The startable service ConsolePagesManagementService allow to manage the page
 * and navigation managmement as creation, update or delete
 * 
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 */
public class ConsolePagesManagementService implements Startable {

  /**
   * The data storage object that is used for the page and navigation CRUD
   */
  private DataStorage dataStorage;

  /**
   * The portal container
   */
  private PortalContainer portalContainer;

  /**
   * The service of portal configuration
   */
  private UserPortalConfigService portalConfigService;

  /**
   * The organisation service that allow to manage the user profiles
   */
  private OrganizationService organisationService;

  /**
   * The bpm console configuration
   */
  private ConsolePortletsConfig consoleConfig;

  /**
   * The logger
   * */
  // private static Logger log =
  // Logger.getLogger(ConsolePagesManagementService.class.getName());
  /**
   * The add plugin method allow to parametrize the console portlet
   * configuration using an the external component plugin system of eXo
   * 
   * @param plugin
   * @throws Exception
   */
  public void addPlugin(ComponentPlugin plugin) throws Exception {
    if (plugin instanceof ConsolePortletsConfigPlugin) {
      /* Get the console portlets configuration from the plugin */
      this.consoleConfig = ((ConsolePortletsConfigPlugin) plugin).getConsolePortletsConfig();
    } else {
      throw new RuntimeException(plugin.getClass().getName() + " is an unknown plugin type.");
    }
  }

  /**
   * Start method for the console pages management service.
   */
  public void start() {
  }

  /**
   * Stop method for the console pages management service.
   */
  public void stop() {
  }

  /**
   * Create the "My Applications" page for the given user. This method create a
   * navigation and a page node that will contain the new user's page
   * 
   * @param user
   * @throws Exception
   */
  public void createUserPages(User user) throws Exception {
    this.portalContainer = PortalContainer.getInstance();
    this.portalConfigService = (UserPortalConfigService) this.portalContainer
                                                             .getComponentInstanceOfType(UserPortalConfigService.class);

    String pageName = "myapplications";
    String id = PortalConfig.USER_TYPE + "::" + user.getUserName() + "::" + pageName;

    if (this.portalConfigService.getPage(id, user.getUserName()) == null) {
      /* Create the user's navigation */
      PageNavigation userNavigation = new PageNavigation();
      userNavigation.setPriority(1);
      userNavigation.setOwnerId(user.getUserName());
      userNavigation.setOwnerType(PortalConfig.USER_TYPE);
      userNavigation.setNodes(new ArrayList<PageNode>());

      /* Create the user's page */
      Page userPage = this.createNewUserPage(user.getUserName());

      /* create the node that will contain the user's page */
      PageNode pageNode = new PageNode();
      pageNode.setPageReference(userPage.getPageId());
      pageNode.setUri("myapp");
      pageNode.setName("myapp");
      pageNode.setLabel("My applications");

      userNavigation.addNode(pageNode);

      /*
       * store the user's navigation and page if the navigation already exist we
       * update it else we create it and store the new user's page
       */
      this.portalConfigService.create(userPage);
      if (this.portalConfigService.getPageNavigation(userNavigation.getId()) == null) {
        this.portalConfigService.create(userNavigation);
      } else {
        this.portalConfigService.update(userNavigation);
      }
    }
  }

  /**
   * Create the "My applications" page of the user. This method allo to set all
   * the necessary informations for this page
   * 
   * @param userName
   * @return The new user page
   * @throws Exception
   */
  private Page createNewUserPage(String userName) throws Exception {
    /* Information preparation */
    Page page = new Page();
    String pageName = "myapplications";
    String id = PortalConfig.USER_TYPE + "::" + userName + "::" + pageName;
    UserACL.Permission permission = new UserACL.Permission();
    permission.setName(userName);
    String[] permissions = {
      permission.getExpression()
    };

    /* Page setting */
    page.setPageId(id);
    page.setOwnerType(PortalConfig.USER_TYPE);
    page.setOwnerId(userName);
    page.setName(pageName);
    page.setTitle("My applications");
    page.setFactoryId(Page.DESKTOP_PAGE);

    page.setAccessPermissions(permissions);
    page.setEditPermission(permission.getExpression());

    ArrayList<Object> childs = new ArrayList<Object>();

    /*
     * By default the page has a welcome widget (available in the default
     * portlets list)
     */
    for (ApplicationDefinition appli : this.consoleConfig.getDefaultPortlets()) {
      Application app = new Application();
      app.setInstanceId(PortalConfig.USER_TYPE 
                        + "#" + userName + ":/" 
                        + appli.getCategory() 
                        + "/" + appli.getName() 
                        + "/" + appli.getName());
      app.setTitle(appli.getDisplayName());

      if (!"".equals(appli.getType())) {
        app.setApplicationType(appli.getType());
      }

      childs.add(app);
    }

    page.setChildren(childs);
    return page;
  }

  /**
   * Update the user's pages when his profile is updated. When adding
   * memberships to a user, this method is called, so the accessible
   * applications to this user are updated according to his rights.
   * 
   * @param the user that the pages have to be updated
   * @throws Exception
   */
  @SuppressWarnings("unchecked")
  public void updateUserPages(Membership membership) throws Exception {
    this.portalContainer = PortalContainer.getInstance();
    this.organisationService = (OrganizationService) this.portalContainer.getComponentInstanceOfType(OrganizationService.class);
    this.portalConfigService = (UserPortalConfigService) this.portalContainer
                                                             .getComponentInstanceOfType(UserPortalConfigService.class);

    /* Get the memberships list of the user */
    Collection<Membership> userMemberships = this.organisationService
                                                 .getMembershipHandler()
                                                 .findMembershipsByUser(membership.getUserName());

    /* The visible portlets list for the connected user */
    Set<String> visiblePortlets = new HashSet<String>();

    if (userMemberships != null) {
      
      /* Add the new membership to the memberships list of the user */
      userMemberships.add(membership);

      /*
       * Add in the portletList the portlets accessible according to the
       * memberships and groups of the user
       */
      for (Membership m : userMemberships) {

        /*
         * The variable accessibleApplications must contain the accessible
         * portlets defined in the configuration file
         */
        List<ApplicationDefinition> accessibleApplications = this.consoleConfig
                                                                 .getAccessiblePortlets(m.getGroupId(), m.getMembershipType());

        /*
         * The variable defaultApplications must contain the default accessible
         * applications
         */
        List<ApplicationDefinition> defaultApplications = this.consoleConfig.getDefaultPortlets();

        /* Get all the names of the accessible/default applications */
        if (accessibleApplications != null) {
          for (ApplicationDefinition appliDef : accessibleApplications) {
            visiblePortlets.add(appliDef.getName());
          }
        }
        if (defaultApplications != null) {
          for (ApplicationDefinition appliDef : defaultApplications) {
            visiblePortlets.add(appliDef.getName());
          }
        }
      }
    }
    /*
     * Get the old "My Application" page for the user to set his new visible
     * portlets according to his new profile
     */
    ArrayList<Page> userPages = this.getPage(membership.getUserName(), null, "myapplications");

    /* The accessible portlets result */
    Map<String, Object> accessiblePortlets = new HashMap<String, Object>();

    /*
     * Get the applications definition from the accessible portlet list Those
     * definition will be used to build the user pages with the correct
     * information of the accessible applications
     */
    Set<ApplicationDefinition> allApplicatons = new HashSet<ApplicationDefinition>();
    for (List<ApplicationDefinition> list : consoleConfig.getAccessiblePortlets().values()) {
      allApplicatons.addAll(list);
    }

    /* Get also the default portlets */
    allApplicatons.addAll(consoleConfig.getDefaultPortlets());

    /* Check if the result is only one page */
    if (userPages.size() == 1) {
      Page myApplicationsPage = userPages.get(0);

      /* According to the user rights we display or not some portlets */
      for (ApplicationDefinition appli : allApplicatons) {
        if (visiblePortlets.contains(appli.getName()) && !accessiblePortlets.containsKey(appli.getName())) {

          /*
           * Construct a page application from the according application from
           * the registry
           */
          Application app = new Application();
          app.setInstanceId(PortalConfig.USER_TYPE 
                            + "#" + membership.getUserName() 
                            + ":/" + appli.getCategory() 
                            + "/" + appli.getName() 
                            + "/" + appli.getName());
          app.setTitle(appli.getDisplayName());
          app.setShowInfoBar(false);
          if (!"".equals(appli.getType())) {
            app.setApplicationType(appli.getType());
          }

          /*
           * The user has the rights to access this portlet so we add this
           * portlet to the user accessibility portlets
           */
          accessiblePortlets.put(appli.getName(), app);
        }
      }
      /* The result list of accessible applications for the user */
      ArrayList<Object> result = new ArrayList<Object>();
      result.addAll(accessiblePortlets.values());

      /* We set the new applications list for the user's page */
      myApplicationsPage.setChildren(result);

      /* Commit on the jcr and flush the cache */
      this.portalConfigService.update(myApplicationsPage);

    } else {
      throw new Exception("[ERROR] The number of \"My Aplications\" pages by user must be only one");
    }
  }

  /**
   * This method allow to find the pages associated to the given owner
   * identificator, type and name by processing a query on the JCR to get from
   * the database the stored configurations.
   * 
   * @param Application component
   * @param The context of the webui request
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @throws Exception
   */
  @SuppressWarnings("unchecked")
  private ArrayList<Page> getPage(final String owner, final String ownerType, final String name) throws Exception {
    this.portalContainer = PortalContainer.getInstance();
    this.dataStorage = (DataStorage) this.portalContainer.getComponentInstanceOfType(DataStorage.class);

    /*
     * Create the query that get the pages of a specific owner, ownerType and
     * name
     */
    ArrayList<Page> pages = new ArrayList<Page>();
    Query<Page> query = new Query<Page>(null, null, null, Page.class);

    query.setOwnerType(ownerType);
    query.setOwnerId(owner);
    query.setName(name);

    /* Get the query result */
    PageList pageList = this.dataStorage.find(query);
    List pagesListe = pageList.getAll();
    for (int i = 0; i < pagesListe.size(); i++) {
      pages.add((Page) pagesListe.get(i));
    }
    return pages;
  }

  /**
   * Delete the applications according to the new profile of the user, after
   * membership removing
   * 
   * @param membership
   * @throws Exception
   */
  @SuppressWarnings("unchecked")
  public void deleteUserApplications(Membership membership) throws Exception {
    this.portalContainer = PortalContainer.getInstance();
    this.organisationService = (OrganizationService) this.portalContainer.getComponentInstanceOfType(OrganizationService.class);
    this.portalConfigService = (UserPortalConfigService) this.portalContainer
                                                             .getComponentInstanceOfType(UserPortalConfigService.class);

    /* This membership list contains yet the membership to delete */
    Collection<Membership> userMemberships = this.organisationService
                                                 .getMembershipHandler()
                                                 .findMembershipsByUser(membership.getUserName());
    
    /*
     * The new visible portlets list according to the new user profile (after
     * membership removing)
     */
    Set<String> visiblePortlets = new HashSet<String>();
    
    if (userMemberships != null) {
      /* So we remove it */
      userMemberships.remove(membership);
  
      /*
       * Add in the visible portlets the accessible ones according to the
       * membership
       */
      for (Membership m : userMemberships) {
        /*
         * The variable accessibleApplications must contain the accessible
         * portlets defined in the configuration file
         */
        List<ApplicationDefinition> accessibleApplications = this.consoleConfig
                                                                 .getAccessiblePortlets(m.getGroupId(), m.getMembershipType());

        /*
         * The variable defaultApplications must contain the default accessible
         * applications
         */
        List<ApplicationDefinition> defaultApplications = this.consoleConfig.getDefaultPortlets();

        /* Get all the names of the accessible/default applications */
        if (accessibleApplications != null) {
          for (ApplicationDefinition appliDef : accessibleApplications) {
            visiblePortlets.add(appliDef.getName());
          }
        }
        if (defaultApplications != null) {
          for (ApplicationDefinition appliDef : defaultApplications) {
            visiblePortlets.add(appliDef.getName());
          }
        }
      }
    }

    /* Get the old "My applications" page of the user */
    ArrayList<Page> userPages = this.getPage(membership.getUserName(), null, "myapplications");

    /* Supress from this page the portlets that becames not accessible */
    if (userPages != null && !userPages.isEmpty()) {

      /* Check if the result is only one page */
      if (userPages.size() == 1) {
        Page myApplicationsPage = userPages.get(0);
        Collection<Object> oldVisibleApplications = (Collection<Object>) myApplicationsPage.getChildren();
        ArrayList<Object> newVisibleApplications = new ArrayList<Object>();

        if (oldVisibleApplications != null) {
          Iterator<Object> iter = oldVisibleApplications.iterator();
          while (iter.hasNext()) {
            Application oldVisibleApplication = (Application) iter.next();

            /*
             * Check if the old visible application is contained in the new
             * visible applications according to the new user profile, if yes
             * the application stay contained in the user's page else we dont
             * add it
             */
            for (String visiblePortlet : visiblePortlets) {
              if (oldVisibleApplication.getInstanceId().contains(visiblePortlet)) {
                newVisibleApplications.add(oldVisibleApplication);
              }
            }
          }
        }

        /*
         * Set the new visible applications in the user "my applications" page
         */
        myApplicationsPage.setChildren(newVisibleApplications);

        /* commit on the jcr and flush the cache */
        this.portalConfigService.update(myApplicationsPage);

      } else {
        throw new Exception("[ERROR] The number of \"My Aplications\" pages by user must be only one");
      }
    }
  }
}
